package de.intevation.gnv.action.sessionmodel;

import de.intevation.gnv.artifactdatabase.objects.ArtifactDescription;
import de.intevation.gnv.artifactdatabase.objects.ArtifactObject;
import de.intevation.gnv.artifactdatabase.objects.ArtifactStatisticsSet;
import de.intevation.gnv.artifactdatabase.objects.OutputMode;

import java.util.Collection;
import java.util.Iterator;

import org.apache.log4j.Logger;

/**
 * The default implementation of <code>SessionModel</code> which stores the
 * current artifact object and the digram options selected by the user.
 *
 * @author <a href="mailto:tim.englich@intevation.de">Tim Englich</a>
 *
 */
public class DefaultSessionModel implements SessionModel {

    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(DefaultSessionModel.class);

    /**
     * The UID of this Class.
     */
    private static final long serialVersionUID = 3431484439985158311L;

    private Collection<ArtifactObject> artifactFactories = null;

    private ArtifactObject selectedArtifactFactory = null;

    private ArtifactObject currentArtifact = null;

    private DiagrammOptions diagrammOptions = null;

    private Collection<ArtifactStatisticsSet> statistics = null;

    /**
     * Constructor
     */
    public DefaultSessionModel() {
        super();
    }


    public void selectArtifactFactory(String artiFactFactoryId) {
        Collection<ArtifactObject> artifactObjects = this
                .getArtifactFactories();
        if (artifactObjects != null) {
            Iterator<ArtifactObject> unselect = artifactObjects.iterator();
            this.selectedArtifactFactory = null;
            while (unselect.hasNext()) {
                unselect.next().setSelected(false);
            }
            Iterator<ArtifactObject> it = artifactObjects.iterator();
            while (it.hasNext()) {
                ArtifactObject obj = it.next();
                if (obj.getId().equals(artiFactFactoryId)) {
                    obj.setSelected(true);
                    this.selectedArtifactFactory = obj;
                    break;
                }
            }
        }
    }


    public void setArtifacteFactories(
                                      Collection<ArtifactObject> artifactFactories) {
        this.artifactFactories = artifactFactories;
    }


    public Collection<ArtifactObject> getArtifactFactories() {
        return this.artifactFactories;
    }


    public ArtifactObject getSelectedArtifactFactory() {
        return this.selectedArtifactFactory;
    }


    public ArtifactObject getCurrentArtifact() {
        return this.currentArtifact;
    }


    public void setCurrentArtifact(ArtifactObject artifact) {
        this.currentArtifact = artifact;
    }


    public DiagrammOptions getDiagrammOptions() {
        return this.diagrammOptions;
    }


    public void setDiagrammOptions(DiagrammOptions diagrammOptions) {
        this.diagrammOptions = diagrammOptions;
    }


    public ArtifactDescription getArtifactDescription() {
        if (this.currentArtifact != null) {
            return (ArtifactDescription) this.currentArtifact;
        }
        return null;
    }


    public OutputMode getOutputMode(String name) {
        ArtifactDescription ad = this.getArtifactDescription();
        if (ad != null) {
            if (ad.getOutputModes() != null) {
                return ad.getOutputModes().get(name);
            }
        }
        return null;
    }


    public void resetModel() {
        log.debug("DefaultSessionModel.resetModel");
        this.selectedArtifactFactory = null;
        this.currentArtifact = null;
        this.diagrammOptions = null;
        this.statistics = null;

    }


    public Collection<ArtifactStatisticsSet> getStatistics() {
        return this.statistics;
    }

    public void setStatistics(Collection<ArtifactStatisticsSet> statistics) {
        this.statistics = statistics;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :