package de.intevation.gnv.action.sessionmodel;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

/**
 * Creates and restores <code>SessionModel</code> objects from <code>
 * HttpServletRequest</code>.
 *
 * @author <a href="mailto:tim.englich@intevation.de">Tim Englich</a>
 */
public class SessionModelFactory {

    public final static String SESSION_MODEL_ID = "de.intevation.gnv.action.sessionmodel.SessionModel.ID";
    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(SessionModelFactory.class);

    /**
     * The singleton Instance of this Factory.
     */
    private static SessionModelFactory instance = null;

    /**
     * Constructor
     */
    private SessionModelFactory() {
        super();
    }

    /**
     * This Method provides an singleton Instance of this Class.
     *
     * @return an singleton Instance of this Class
     */
    public synchronized static SessionModelFactory getInstance() {
        if (instance == null) {
            instance = new SessionModelFactory();
        }
        return instance;
    }

    /**
     * Getting the ArtifactDatabaseClient
     *
     * @return the ArtifactDatabaseClient
     */
    public SessionModel getSessionModel(HttpServletRequest request) {
        synchronized (request) {
            SessionModel sm = null;
            Object obj = request.getSession().getAttribute(SESSION_MODEL_ID);
            if (obj instanceof SessionModel) {
                sm = (SessionModel) obj;
            } else {
                sm = new DefaultSessionModel();
                request.getSession().setAttribute(SESSION_MODEL_ID, sm);
            }

            return sm;
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :