package de.intevation.gnv.artifactdatabase.objects;

import java.io.Serializable;

import java.util.Collection;
import java.util.Map;

import org.w3c.dom.Node;

/**
 * This interface describes basic methods to describe the user interface of an
 * artifact.
 *
 * @author <a href="mailto:tim.englich@intevation.de">Tim Englich</a>
 */
public interface ArtifactDescription extends Serializable {

    /**
     * Retrieves the xml node of the current user interface.
     *
     * @return the currentUI
     */
    public Node getCurrentUI();

    /**
     * Set the xml node of the current user interface.
     *
     * @param currentUI the currentUI to set
     */
    public void setCurrentUI(Node currentUI);

    /**
     * Set the xml node of the possible output modes.
     *
     * @param currentOut the currentOut to set
     */
    public void setCurrentOut(Node currentOut);

    /**
     * Retrieves a collection of reachable states.
     *
     * @return the reachableStates
     */
    public Collection<String> getReachableStates();

    /**
     * Set the reachable states.
     *
     * @param reachableStates the reachableStates to set
     */
    public void setReachableStates(Collection<String> reachableStates);

    /**
     * Retrieves the current state.
     *
     * @return the currentState
     */
    public String getCurrentState();

    /**
     * Set the current state.
     *
     * @param currentState the currentState to set
     */
    public void setCurrentState(String currentState);

    /**
     * Get a collection of input parameters.
     *
     * @return the reachableStates
     */
    public Collection<String> getInputParameter();

    /**
     * Set a collection of input parameters.
     *
     * @param inputParameter
     */
    public void setInputParameter(Collection<String> inputParameter);

    /**
     * Retrieves a map of available output modes.
     *
     * @return the available output modes.
     */
    public Map<String, OutputMode> getOutputModes();

    /**
     * Retrieves a collection of available output modes.
     *
     * @return the available output modes.
     */
    public Collection<OutputMode> getOutputModesAsCollection();

    /**
     * Set the available output modes.
     *
     * @param outputModes A map that contains output modes.
     */
    public void setOutputModes(Map<String, OutputMode> outputModes);
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :