package de.intevation.gnv.artifactdatabase.objects;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * This Class is an Extention of the Class ArtifactFactory.
 * Its only job is to store optional Parameters that might be
 * used to instantiate a new Artifact.
 * @author <a href="mailto:tim.englich@intevation.de">Tim Englich</a>
 */
public class ParametrizedArtifactFactory
extends ArtifactFactory implements ParametrizedArtifactObject {

    /**
     * The UID of this Class
     */
    private static final long serialVersionUID = 4516368434861819032L;

    /**
     * The Parameters that belongs to the Artifactfactory and might be used
     * during the Initializationprocess of an Artifact.
     */
    private Map<String, Collection<String>> parameters = null;

    /**
     * Constructor
     * @param name the name of the factory
     * @param description the description of the Factory
     * @param dataBaseUrl the URL where the Factory could be reached.
     */
    public ParametrizedArtifactFactory(String name, String description,
                                       String dataBaseUrl) {
        super(name, description, dataBaseUrl);
        this.parameters = new HashMap<String, Collection<String>>();
    }

    public Map<String, Collection<String>> getParameters() {
        return parameters;
    }

    public void addParameters(String key, Collection<String> values) {
        this.parameters.put(key, values);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :